"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Action = void 0;
// external
const filedirname_1 = __importDefault(require("filedirname"));
// builtin
const path_1 = require("path");
const os = __importStar(require("os")); // { availableParallelism } not available on node.js <18
const cluster_1 = __importDefault(require("cluster"));
// prepare
const [file, dir] = (0, filedirname_1.default)();
const workerPath = (0, path_1.join)(dir, 'worker.js');
var Action;
(function (Action) {
    Action["readdir"] = "readdir";
    Action["stat"] = "stat";
})(Action || (exports.Action = Action = {}));
async function readdirCluster({ directory, iterator, retryDelay, }) {
    // handle the messages
    const resolvers = new Map();
    let resolverCursor = 0;
    function sendMessageAndWait(action, path) {
        return new Promise(function (resolve, reject) {
            ++resolverCursor;
            resolvers.set(resolverCursor, resolve);
            const message = { id: resolverCursor, action, path };
            function sendAndTimeout() {
                const worker = nextWorker();
                if (!worker) {
                    reject(new Error('no workers available'));
                    return;
                }
                worker.send(message);
                setTimeout(function () {
                    const resolver = resolvers.has(message.id);
                    // console.log('yo', message.id, resolver)
                    if (resolver) {
                        // console.log('trying again', message.id, worker)
                        sendAndTimeout();
                    }
                }, retryDelay !== null && retryDelay !== void 0 ? retryDelay : 10);
            }
            sendAndTimeout();
        });
    }
    function receiveMessageAndResume(message) {
        const resolver = resolvers.get(message.id);
        // console.log('resolve', message.id, resolver, resolvers.keys())
        if (!resolver) {
            // console.log('already completed', message.id)
        }
        else {
            resolvers.delete(message.id);
            resolver(message);
        }
    }
    // handle the workers
    const workersCount = os.availableParallelism
        ? os.availableParallelism()
        : os.cpus().length || 4;
    const workers = [];
    function openWorker() {
        return new Promise(function (resolve, reject) {
            const worker = cluster_1.default.fork();
            workers.push(worker);
            worker.on('online', resolve);
            worker.on('message', receiveMessageAndResume);
            // worker.on('error', () => {}) // ignore, as it is likely just a complaint about a retry sending a message once we have disconnected
            // worker.on('disconnect', console.error.bind(console, 'worker disconnect'))
            // worker.on('exit', console.error.bind(console, 'worker exit'))
        });
    }
    async function openWorkers() {
        ;
        (cluster_1.default.setupPrimary || cluster_1.default.setupMaster)({ exec: workerPath });
        for (let i = 0; i < workersCount; i++)
            await openWorker();
    }
    function closeWorkers() {
        // console.log('closing workers')
        for (const worker of workers) {
            worker.destroy();
        }
    }
    function nextWorker() {
        const worker = workers.shift();
        if (!worker) {
            // ignore, we are shutting down
        }
        else if (worker.isConnected()) {
            // re-add the worker to make it available
            workers.push(worker);
        }
        else {
            // the worker died, don't re-add it
        }
        return worker || null;
    }
    // readdir
    async function readdir(directory) {
        // prepare
        const results = [];
        const message = await sendMessageAndWait(Action.readdir, directory);
        if (message.error)
            return Promise.reject(message.error);
        await Promise.all(message.data.map(async (file) => {
            const path = (0, path_1.join)(directory, file);
            const stat = await sendMessageAndWait(Action.stat, path);
            if (stat.error)
                return Promise.reject(stat.error);
            const iterate = iterator && iterator(path, file, stat.data);
            if (iterate === false)
                return;
            results.push(path);
            if (stat.data.directory) {
                const subpaths = await readdir(path);
                results.push(...subpaths);
            }
        }));
        return results;
    }
    // process
    try {
        await openWorkers();
        const results = await readdir(directory);
        closeWorkers();
        return results;
    }
    catch (error) {
        closeWorkers();
        return Promise.reject(error);
    }
}
exports.default = readdirCluster;
