/// <reference types="node" />
import { Stats } from 'fs';
export declare enum Action {
    readdir = "readdir",
    stat = "stat"
}
export type Paths = Array<string>;
export type Stat = Omit<Stats, 'isFile' | 'isDirectory' | 'isBlockDevice' | 'isCharacterDevice' | 'isSymbolicLink' | 'isFIFO' | 'isSocket'> & {
    directory: boolean;
};
export interface MessageRequest {
    id: number;
    action: Action;
    path: string;
    error?: undefined;
    data?: undefined;
}
export interface MessageFailure {
    id: number;
    action: Action;
    path: string;
    error: Error;
    data: undefined;
}
export interface MessageReaddir {
    id: number;
    action: Action.readdir;
    path: string;
    error: undefined;
    data: Paths;
}
export interface MessageStat {
    id: number;
    action: Action.stat;
    path: string;
    error: undefined;
    data: Stat;
}
export type Message = MessageRequest | MessageFailure | MessageReaddir | MessageStat;
export type Iterator = (path: string, filename: string, stat: Stat) => boolean | void;
export interface Options {
    /** The directory to read */
    directory: string;
    /** A custom iterator, return false to ignore the path. */
    iterator?: Iterator;
    /** How long in milliseconds until we try the path again? Defaults to 10 */
    retryDelay?: number;
}
export default function readdirCluster({ directory, iterator, retryDelay, }: Options): Promise<Paths>;
//# sourceMappingURL=index.d.ts.map